ExtendedVehicleEntry = {}

function ExtendedVehicleEntry:update()
    ExtendedVehicleEntry:TargetRaycaster();
end;

function ExtendedVehicleEntry:HasTarget()
    if ExtendedVehicleEntry.TargetVehicle == nil or ExtendedVehicleEntry.OwnerVehicle == nil then return false; end;
    return true;
end

function ExtendedVehicleEntry:GetTarget()
    return ExtendedVehicleEntry.OwnerVehicle;
end

function ExtendedVehicleEntry:TargetRaycaster()
    if g_currentMission.player == nil then return; end
    if g_currentMission.controlledVehicle ~= nil then return; end

    ExtendedVehicleEntry.TargetVehicle = nil;

    local x, y, z = localToWorld(g_currentMission.player.cameraNode, 0, 0, 1.0);
    local dx, dy, dz = localDirectionToWorld(g_currentMission.player.cameraNode, 0, 0, -1);
    raycastAll(x, y, z, dx, dy, dz, "raycastCallback", 7, self);

    ExtendedVehicleEntry.OwnerVehicle = ExtendedVehicleEntry:GetOwnerEnterable(ExtendedVehicleEntry.TargetVehicle);
end

function ExtendedVehicleEntry:GetOwnerEnterable(IVehicle)
    if IVehicle == nil then return nil; end;

    if IVehicle.spec_enterable ~= nil then
        return IVehicle;
    end;

    for _, vehicle in ipairs (g_currentMission.vehicles) do
        if(vehicle.spec_enterable) then
        local ChildVehicles = vehicle:getChildVehicles()
        for i=1, #ChildVehicles do
            if ChildVehicles[i] == IVehicle then
                return vehicle;
            end;
        end
    end
    end

    return nil;
end

function ExtendedVehicleEntry:raycastCallback(NodeTarget)
    if NodeTarget == g_currentMission.player.rootNode then
        return true;
    end

    for _, vehicle in ipairs (g_currentMission.vehicles) do
        if vehicle.spec_fillUnit ~= nil and vehicle.spec_fillUnit:getFillUnitIndexFromNode(NodeTarget) ~= nil then
            ExtendedVehicleEntry.TargetVehicle = vehicle;
            return false;
        end;

        if ExtendedVehicleEntry:IsNodeParent(vehicle.rootNode,NodeTarget) or ExtendedVehicleEntry:IsNodeParent(vehicle.rootNode,getParent(NodeTarget)) then
            ExtendedVehicleEntry.TargetVehicle = vehicle;
            return false;
        end;
    end


    return true;
end;

function ExtendedVehicleEntry:IsNodeParent(Parent,Children)

    if Parent == Children then return true; end;

    for i = 0, getNumOfChildren(Parent)-1 do
        local Child = getChildAt(Parent, i);
        if Child == Children then return true; end;
    end

    return false;
end;

function PlayerUpdateActionEvents(self, superFunc)
        -- light
        local isDark = not g_currentMission.environment.isSunOn
        local eventIdToggleLight = self.inputInformation.registrationList[InputAction.TOGGLE_LIGHTS_FPS].eventId
        if self.playerStateMachine:isAvailable("useLight") and isDark then
            g_inputBinding:setActionEventTextVisibility(eventIdToggleLight, isDark and self.model:getHasTorch())
        end
    
        local stateSwitchHandTool = false
        local stateActivateHandTool = false
        local stateInteract = false
        local stateAnimalPet = false
        local stateEnter = false
        local stateThrowObject = false
        local stateObjectLeftRight = false
        local stateObjectUpDown = false
    
        -- tools
        if self.playerStateMachine:isAvailable("cycleHandtool") then
            stateSwitchHandTool = true
        end
    
        if self:hasHandtoolEquipped() then
            stateActivateHandTool = true
            self.playerStateMachine:isAvailable("activateObject")
        else
            if self.playerStateMachine:isAvailable("throw") then
                stateThrowObject = true
            end
    
            if self.isCarryingObject then
                stateObjectLeftRight = true
                stateObjectUpDown = true
            end
    
            if self.playerStateMachine:isAvailable("animalPet") then
                local eventIdActivateObject = self.inputInformation.registrationList[InputAction.ANIMAL_PET].eventId
                stateAnimalPet = true
                g_inputBinding:setActionEventText(eventIdActivateObject, g_i18n:getText("action_petAnimal"))
            end
    
            local eventIdInteract = self.inputInformation.registrationList[InputAction.INTERACT].eventId
            if self.playerStateMachine:isAvailable("drop") then
                g_inputBinding:setActionEventText(eventIdInteract, g_i18n:getText("action_dropObject"))
                stateInteract = true
            elseif self.playerStateMachine:isAvailable("pickup") then
                g_inputBinding:setActionEventText(eventIdInteract, g_i18n:getText("action_pickUpObject"))
                stateInteract = true
            elseif self.playerStateMachine:isAvailable("animalInteract") or self.playerStateMachine:isActive("animalInteract") then
                local animalInteractState = self.playerStateMachine:getState("animalInteract")
                local animalInteractText = string.format(g_i18n:getText("action_interactAnimal"), animalInteractState.interactText)
                g_inputBinding:setActionEventText(eventIdInteract, animalInteractText)
                stateInteract = true
            end
        end
    
        -- enter vehicle or ride animal
        self.canRideAnimal = self.playerStateMachine:isAvailable("animalRide")
        self.canEnterVehicle = g_currentMission.interactiveVehicleInRange and g_currentMission.interactiveVehicleInRange:getIsEnterable()
        ExtendedVehicleEntry.DummyEntry = false;
        if ExtendedVehicleEntry:HasTarget() == true then
            ExtendedVehicleEntry.DummyEntry = true;
             self.canEnterVehicle = ExtendedVehicleEntry:HasTarget(); 
             g_currentMission.interactiveVehicleInRange = ExtendedVehicleEntry:GetTarget(); 
        end;
        local vehicleIsRideable = self.canEnterVehicle and SpecializationUtil.hasSpecialization(Rideable, g_currentMission.interactiveVehicleInRange.specializations)
        local eventIdEnter = self.inputInformation.registrationList[InputAction.ENTER].eventId
        if self.canEnterVehicle and not vehicleIsRideable then
            g_inputBinding:setActionEventText(eventIdEnter, g_i18n:getText("button_enterVehicle"))
            stateEnter = true
        elseif self.canRideAnimal or vehicleIsRideable then
            local rideableName = ""
            if self.canRideAnimal then
                local rideState = self.playerStateMachine:getState("animalRide")
                rideableName = rideState:getRideableName()
            elseif vehicleIsRideable then
                rideableName = g_currentMission.interactiveVehicleInRange:getFullName()
            end
            g_inputBinding:setActionEventText(eventIdEnter, string.format(g_i18n:getText("action_rideAnimal"), rideableName))
            stateEnter = true
        end
    
        -- first disable all inactive inputs to avoid conflicts if not active in this frame
        self:disableInput(InputAction.SWITCH_HANDTOOL, stateSwitchHandTool)
        self:disableInput(InputAction.ACTIVATE_HANDTOOL, stateActivateHandTool)
        self:disableInput(InputAction.INTERACT, stateInteract)
        self:disableInput(InputAction.ANIMAL_PET, stateAnimalPet)
        self:disableInput(InputAction.ENTER, stateEnter)
        self:disableInput(InputAction.THROW_OBJECT, stateThrowObject)
        self:disableInput(InputAction.ROTATE_OBJECT_LEFT_RIGHT, stateObjectLeftRight)
        self:disableInput(InputAction.ROTATE_OBJECT_UP_DOWN, stateObjectUpDown)
    
        self:setInputState(InputAction.SWITCH_HANDTOOL, stateSwitchHandTool)
        self:setInputState(InputAction.ACTIVATE_HANDTOOL, stateActivateHandTool)
        self:setInputState(InputAction.INTERACT, stateInteract)
        self:setInputState(InputAction.ANIMAL_PET, stateAnimalPet)
        self:setInputState(InputAction.ENTER, stateEnter)
        self:setInputState(InputAction.THROW_OBJECT, stateThrowObject)
        self:setInputState(InputAction.ROTATE_OBJECT_LEFT_RIGHT, stateObjectLeftRight)
        self:setInputState(InputAction.ROTATE_OBJECT_UP_DOWN, stateObjectUpDown)
    
    
        -- debug movements
        local eventIdDebugFlyToggle = self.inputInformation.registrationList[InputAction.DEBUG_PLAYER_ENABLE].eventId
        g_inputBinding:setActionEventActive(eventIdDebugFlyToggle, g_flightModeEnabled)
        local eventIdDebugFlyUpDown = self.inputInformation.registrationList[InputAction.DEBUG_PLAYER_UP_DOWN].eventId
        g_inputBinding:setActionEventActive(eventIdDebugFlyUpDown, g_flightModeEnabled)
end

function PlayerOnInputEnter(self, superFunc, _, inputValue)
    if g_time > g_currentMission.lastInteractionTime + 200 then
        if g_currentMission.interactiveVehicleInRange and g_currentMission.accessHandler:canFarmAccess(self.farmId, g_currentMission.interactiveVehicleInRange) then
            g_currentMission.interactiveVehicleInRange:interact()
        elseif self.canRideAnimal then
            self.playerStateMachine:activateState("animalRide")
        end
    end

    if ExtendedVehicleEntry.DummyEntry == true then
        g_currentMission:requestToEnterVehicle(ExtendedVehicleEntry:GetTarget());
    end
end

Player.onInputEnter = Utils.overwrittenFunction(Player.onInputEnter, PlayerOnInputEnter);
Player.updateActionEvents = Utils.overwrittenFunction(Player.updateActionEvents, PlayerUpdateActionEvents);
addModEventListener(ExtendedVehicleEntry);
